#include <stdlib.h>
#include <stdio.h>

#include <ros/ros.h>
#include <cereal_port/CerealPort.h>
#include <geometry_msgs/TransformStamped.h>
#include <tf/transform_broadcaster.h>

#include <sr_robot_msgs/sendupdate.h>
#include <sr_robot_msgs/joint.h>

#include "std_msgs/MultiArrayLayout.h"
#include "std_msgs/MultiArrayDimension.h"

#include "std_msgs/Int32MultiArray.h"

#define REPLY_SIZE 250
#define TIMEOUT 2000

geometry_msgs::Quaternion AccToQuat(int x, int y, int z) {
	
	float roll =  x * (3.14169/180);
	float pitch = y * (3.14169/180);
	float yaw =   z * (3.14169/180);
	
	//ROS_INFO("Sensor INFO: %d %d %d | %f %f %f", x,y,z,roll,pitch,yaw);

	////Just some demo values to see the cube spinning in rviz
	//if (roll>3) {
		//pitch +=0.01;
		//}
		//else {roll+=0.01; }

	/** 
	* Convert roll, pitch and yaw into a Quaternion
	*/			
	tf::Quaternion q;
	q.setRPY(roll, pitch, yaw);
	
	geometry_msgs::Quaternion quat;
	tf::quaternionTFToMsg(q, quat); // convert to proper format (geometry_msgs)
	
	return quat;
	
	
}

int main(int argc, char** argv)
{

	/**
	* The ros::init() function needs to see argc and argv so that it can perform
	* any ROS arguments and name remapping that were provided at the command line. For programmatic
	* remappings you can use a different version of init() which takes remappings
	* directly, but for most command-line programs, passing argc and argv is the easiest
	* way to do it.  The third argument to init() is the name of the node.
	*
	* You must call one of the versions of ros::init() before using any other
	* part of the ROS system.
	*/
    ros::init(argc, argv, "serial_iTH_node");
    
		
	/**
	* NodeHandle is the main access point to communications with the ROS system.
	* The first NodeHandle constructed will fully initialize this node, and the last
	* NodeHandle destructed will close down the node.
	*/
    ros::NodeHandle n;


	/**
	* The advertise() function is how you tell ROS that you want to
	* publish on a given topic name. This invokes a call to the ROS
	* master node, which keeps a registry of who is publishing and who
	* is subscribing. After this advertise() call is made, the master
	* node will notify anyone who is trying to subscribe to this topic name,
	* and they will in turn negotiate a peer-to-peer connection with this
	* node.  advertise() returns a Publisher object which allows you to
	* publish messages on that topic through a call to publish().  Once
	* all copies of the returned Publisher object are destroyed, the topic
	* will be automatically unadvertised.
	*
	* The second parameter to advertise() is the size of the message queue
	* used for publishing messages.  If messages are published more quickly
	* than we can send them, the number here specifies how many messages to
	* buffer up before throwing some away.
	*/
 	ros::Publisher serial_glove_pub = n.advertise<geometry_msgs::TransformStamped>("/cubepose", 192);
 	ros::Publisher RPY_pub = n.advertise<std_msgs::Int32MultiArray>("/H4joints", 192);
	
	
	geometry_msgs::TransformStamped cubepose;


	/** 
	* Start the c++ object of our serial connection.
	* Configuration of the connection parameters 
	*/
	cereal::CerealPort device;
        
    // Change the next line according to your port name and baud rate
    try
    { 
		device.open("/dev/ttyUSB0", 115200); 
	}
    catch(cereal::Exception& e)
    {
        ROS_FATAL("Failed to open the serial port!!!");
        ROS_BREAK();
    }
    ROS_INFO("The serial port is opened.");


	/** 
	* Start the array of values to be read from serial connection 
	*/
	char reply[REPLY_SIZE];
	char reply2[1000];

	
	
	/**
	* ros::Rate is the control parameter for the while cicle
	* It is measured in Hz (times/s)
	*/
    ros::Rate r(5);
    
    ROS_INFO("The node now starts reading from serial port.");

    while(ros::ok())
    {
		
        
		int pitch = 0;	
		
		/*
		int yaw = 0;
		int roll = 0;		
		
		yaw = 0;
		yaw = yaw +reply[13];
		yaw = (yaw<<8);
		int yaw2 = 0; yaw2 = (unsigned char) reply[14];
		yaw = yaw | yaw2;
		pitch = reply[15];
		roll = reply[16];
          
      

		roll = (roll);
		pitch = (pitch);
		yaw = (yaw)/10;	
		
		ROS_INFO("ID: %d\tAngles: %d %d %d", reply[0],yaw, pitch, roll);
		*/

        /** Send message over the serial port */
        char buffer2[3] = {0x70, 0x00, 0xe0};
        device.write(buffer2, 3);

        /** Get the reply from serial port */
        try
        {
			device.readBytes(reply2, 3*17, 100);
		}
        catch(cereal::TimeoutException& e)
        {
            ROS_ERROR("2Timeout!");
        }

		int idx;
		 idx=0;	ROS_INFO("ID: %d\tAngles: %d %d %d", reply2[0+idx*17],0, (int) reply2[15+idx*17], (int) reply2[16+idx*17]);
		 idx=1;	ROS_INFO("ID: %d\tAngles: %d %d %d", reply2[0+idx*17],0, reply2[15+idx*17], reply2[16+idx*17]);
		 idx=2;	ROS_INFO("ID: %d\tAngles: %d %d %d", reply2[0+idx*17],0, reply2[15+idx*17], reply2[16+idx*17]);
			
			std_msgs::Int32MultiArray array;
		//Clear array
		array.data.clear();
		array.data.push_back((int) reply2[15+0*17]);
		array.data.push_back((int) reply2[15+1*17]);
		array.data.push_back((int) reply2[15+2*17]);
		//Publish array
		RPY_pub.publish(array);
			
			
			
		//	geometry_msgs::Quaternion quat = AccToQuat(roll, -pitch, -yaw);
			
			
			/**
			* Prepare cubepose ROS msg for publishing
			*/
			/*
			cubepose.header.stamp = ros::Time::now();   
			cubepose.header.frame_id = "base_link";
			cubepose.child_frame_id = "cube_frame";
			//cubepose.child_frame_id = "";
			cubepose.transform.translation.x = 0;
			cubepose.transform.translation.y = 0;
			cubepose.transform.translation.z = 0;
			
			cubepose.transform.rotation = quat;
			
			
			/**
			* The publish() function is how you send messages. The parameter
			* is the message object. The type of this object must agree with the type
			* given as a template parameter to the advertise<>() call, as was done
			* in the constructor above.
			*/
			/*
			serial_glove_pub.publish(cubepose);		// cube pose publish
				*/
		/**
		* Calling ros::spinOnce() here is not necessary for this simple
		* program, because we are not receiving any callbacks. However,
		* if you were to add a subscription into this application, and
		* did not have ros::spinOnce() here, your callbacks would never 
		* get called. So, add it for good measure.
		*/
        ros::spinOnce();
        
        /**
        * Now we use the ros::Rate object to sleep for the time
        * remaining to let us hit our 10hz publish rate.
        */
        r.sleep();
    }   
    
    return 0;
}
