#include <stdlib.h>
#include <stdio.h>

#include <ros/ros.h>

#include <geometry_msgs/TransformStamped.h>
#include <tf/transform_broadcaster.h>

#include <sr_robot_msgs/sendupdate.h>
#include <sr_robot_msgs/joint.h>

#include "std_msgs/MultiArrayLayout.h"
#include "std_msgs/MultiArrayDimension.h"

#include "std_msgs/Int32MultiArray.h"


int Arr[90];
void arrayCallback(const std_msgs::Int32MultiArray::ConstPtr& array)
{

	int i = 0;
	// print all the remaining numbers
	for(std::vector<int>::const_iterator it = array->data.begin(); it != array->data.end(); ++it)
	{
		Arr[i] = *it;
		ROS_INFO("Arr[%d]:%d",i,Arr[i]);
		i++;
	}
	printf("\n");
	
	///publish to the shadowhand sendupdate topic
	static ros::NodeHandle n;
	static ros::Publisher shadowhand_pub = n.advertise<sr_robot_msgs::sendupdate> ("/srh/sendupdate", 5);
	static ros::Publisher shadowarm_pub = n.advertise<sr_robot_msgs::sendupdate> ("/sr_arm/sendupdate", 5);
	
	sr_robot_msgs::joint hand_joint;
	sr_robot_msgs::joint arm_joint;
	sr_robot_msgs::sendupdate pub;


    /// Generate sendupdate message for the hand
	std::vector<std::string> hand_joints_names;
    unsigned int number_hand_joints = 3;
    hand_joints_names.resize(number_hand_joints);

	hand_joints_names[0] = "FFJ0";
    hand_joints_names[1] = "MFJ0";
    hand_joints_names[2] = "RFJ0";
    
    std::vector<sr_robot_msgs::joint> hand_table(number_hand_joints);
    for(unsigned int i = 0; i < number_hand_joints; ++i )
    {
        hand_joint.joint_name = hand_joints_names[i];
        hand_joint.joint_target = Arr[i];
        hand_table[i] = hand_joint;
    }
    pub.sendupdate_length = number_hand_joints;
    pub.sendupdate_list = hand_table;
    shadowhand_pub.publish(pub);




    /// Generate sendupdate message for the arm
	std::vector<std::string> arm_joints_names;
    unsigned int number_arm_joints = 4;
    arm_joints_names.resize(number_arm_joints);

	arm_joints_names[0] = "ElbowJRotate";
    arm_joints_names[1] = "ElbowJSwing";
    arm_joints_names[2] = "ShoulderJRotate";
    arm_joints_names[2] = "ShoulderJSwing";
    
    Arr[0] = 45;
    Arr[1] = 0;
    Arr[2] = 45;
    Arr[3] = 0;
    
    std::vector<sr_robot_msgs::joint> arm_table(number_arm_joints);
    for(unsigned int i = 0; i < number_arm_joints; ++i )
    {
        arm_joint.joint_name = arm_joints_names[i];
        arm_joint.joint_target = Arr[i];
        arm_table[i] = arm_joint;
    }
    pub.sendupdate_length = number_arm_joints;
    pub.sendupdate_list = arm_table;
    shadowarm_pub.publish(pub);

	
	return;

}

  /*    
	joints_names:
	"THJ1";
	"THJ2";
	"THJ3";
	"THJ4";
	"THJ5";
	"FFJ0";
	"FFJ3";
	"FFJ4";
	"MFJ0";
	"MFJ3";
	"MFJ4";
	"RFJ0";
	"RFJ3";
	"RFJ4";
	"LFJ0";
	"LFJ3";
	"LFJ4";
	"LFJ5";
	"WRJ1";
	"WRJ2";
    
	"ElbowJRotate"
	"ElbowJSwing"
	"ShoulderJRotate"
	"ShoulderJSwing"

    */



void cube_callback(geometry_msgs::TransformStamped msg){ 


	/**
	* Sending the message retrieved from the topic into the relevant 
	* transformation reference
	*/
    static tf::TransformBroadcaster pose_broadcaster;
    pose_broadcaster.sendTransform(msg); 
}


int main(int argc, char** argv)
{

	/**
	* The ros::init() function needs to see argc and argv so that it can perform
	* any ROS arguments and name remapping that were provided at the command line. For programmatic
	* remappings you can use a different version of init() which takes remappings
	* directly, but for most command-line programs, passing argc and argv is the easiest
	* way to do it.  The third argument to init() is the name of the node.
	*
	* You must call one of the versions of ros::init() before using any other
	* part of the ROS system.
	*/
    ros::init(argc, argv, "serial_iTH_visualization_node");
    
		
	/**
	* NodeHandle is the main access point to communications with the ROS system.
	* The first NodeHandle constructed will fully initialize this node, and the last
	* NodeHandle destructed will close down the node.
	*/
    ros::NodeHandle n;

	
	ROS_INFO("Reading messages from the topic and sending them to callback... .");

 	
 	/**
 	* In the subscribe() function we get the information from the
 	* previously created topic. The third parameter of the function
 	* is the callback function.
 	*/
 	ros::Subscriber serial_glove_sub = n.subscribe<geometry_msgs::TransformStamped>("/cubepose", 75, cube_callback);
	ros::Subscriber sub3 = n.subscribe("/H4joints", 100, arrayCallback);

	/**
	* Calling ros::spin() allows our callback to be called
	* everytime. In this case everytime there is a new message in 
	* the subscribed node the callback will be ..called :)
	*/
	ros::spin();
			
	return(0);
	
}
